/* Fo
 * fo-datatype-string.c: String datatype
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-string.c,v 1.3 2003/07/02 10:43:35 tonygraham Exp $
 *
 * See Copyright for the status of this software.
 */

#include "libfo/fo-utils.h"
#include "fo-datatype.h"
#include "fo-datatype-private.h"
#include "fo-string.h"


enum {
  PROP_0,
  PROP_VALUE
};

struct _FoString
{
  FoDatatype parent_instance;

  gchar *value;
};

struct _FoStringClass
{
  FoDatatypeClass parent_class;
  
};

static void fo_string_init         (FoString      *string);
static void fo_string_class_init   (FoStringClass *klass);
static void fo_string_set_property (GObject       *object,
				    guint          prop_id,
				    const GValue  *value,
				    GParamSpec    *pspec);
static void fo_string_get_property (GObject       *object,
				    guint          prop_id,
				    GValue        *value,
				    GParamSpec    *pspec);
static void fo_string_finalize     (GObject       *object);

FoDatatype* fo_string_copy(FoDatatype *datatype);
gchar*      fo_string_sprintf (FoObject *object);

static gpointer parent_class;

GType
fo_string_get_type (void)
{
  static GType object_type = 0;

  if (!object_type)
    {
      static const GTypeInfo object_info =
      {
        sizeof (FoStringClass),
        (GBaseInitFunc) NULL,
        (GBaseFinalizeFunc) NULL,
        (GClassInitFunc) fo_string_class_init,
        NULL,           /* class_finalize */
        NULL,           /* class_data */
        sizeof (FoString),
        0,              /* n_preallocs */
        (GInstanceInitFunc) fo_string_init,
      };
      
      object_type = g_type_register_static (FO_TYPE_DATATYPE,
                                            "FoString",
                                            &object_info, 0);
    }
  
  return object_type;
}

static void
fo_string_init (FoString *string)
{
  string->value = NULL;
}

static void
fo_string_class_init (FoStringClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  
  parent_class = g_type_class_peek_parent (klass);
  
  object_class->finalize = fo_string_finalize;

  object_class->set_property = fo_string_set_property;
  object_class->get_property = fo_string_get_property;

  FO_OBJECT_CLASS (klass)->sprintf = fo_string_sprintf;

  g_object_class_install_property (object_class,
                                   PROP_VALUE,
                                   g_param_spec_string ("value",
						     _("Value"),
						     _("String value"),
							NULL,
						     G_PARAM_READWRITE));

  FO_DATATYPE_CLASS (klass)->copy = fo_string_copy;
}

static void
fo_string_finalize (GObject *object)
{
  FoString *string;

  string = FO_STRING (object);

  g_free(string->value);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}


static void
fo_string_set_property (GObject         *object,
                         guint            prop_id,
                         const GValue    *value,
                         GParamSpec      *pspec)
{
  FoDatatype *string;

  string = FO_DATATYPE (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      fo_string_set_value (string, g_value_get_string (value));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

static void
fo_string_get_property (GObject         *object,
                         guint            prop_id,
                         GValue          *value,
                         GParamSpec      *pspec)
{
  FoDatatype *string;

  string = FO_DATATYPE (object);

  switch (prop_id)
    {
    case PROP_VALUE:
      g_value_set_string (value, fo_string_get_value (string));
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/**
 * fo_string_new:
 * 
 * Creates a new #FoString initialized to default value.
 * 
 * Return value: the new #FoString
 **/
FoDatatype *
fo_string_new (void)
{
  FoDatatype *string;

  string = FO_DATATYPE (g_object_new (fo_string_get_type (), NULL));
  
  return string;
}

/* value */
gchar*
fo_string_get_value (FoDatatype *string)
{
  g_return_val_if_fail (string != NULL, NULL);
  g_return_val_if_fail (FO_IS_STRING (string), NULL);

  return g_strdup (FO_STRING (string)->value);
}

void
fo_string_set_value (FoDatatype   *string,
		     const gchar*  new_value)
{
  g_return_if_fail (string != NULL);
  g_return_if_fail (FO_IS_STRING (string));

  FO_STRING (string)->value = g_strdup (new_value);
  /*g_object_notify(G_OBJECT(string), "value");*/
}

FoDatatype*
fo_string_copy (FoDatatype *datatype)
{
  FoDatatype *string;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_STRING (datatype), NULL);

  string = fo_string_new ();
  FO_STRING (string)->value = g_strdup (FO_STRING (datatype)->value);

  return (string);
}

gchar*
fo_string_sprintf (FoObject *object)
{
  g_return_val_if_fail (object != NULL, NULL);
  g_return_val_if_fail (FO_IS_STRING (object), NULL);

  return (g_strdup_printf ("\"%s\"", FO_STRING (object)->value));
}
