/* Fo
 * fo-property-util.c: Utility functions for properties.
 *
 * Copyright (C) 2001 Sun Microsystems
 *
 * $Id: fo-property-util.c,v 1.1 2003/02/28 01:50:01 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include <string.h>
#include "fo-property-util.h"

/**
 * fo_property_border_get_color_initial:
 * 
 * Get the initial color value for borders.
 * 
 * Return value: #FoDatatype for initial border color.
 **/
FoDatatype*
fo_property_get_color_initial (void)
{
  return fo_color_get_color_black ();
}

/**
 * fo_property_resolve_color_enum:
 * @token:   Token from the XML attribute value to be evaluated as an
 *           enumeration token.
 * @context: #FoContext object from which to possibly inherit values.
 * @error:   Information about any error that has occurred.
 * 
 * Compare @token against the enumeration tokens that are valid for the
 * current FO property.  If @token is valid, returns either an #FoEnum datatype
 * representing the enumeration token or a different datatype representing
 * the enumeration token's resolved value.  If @token is not valid,
 * sets @error and returns NULL.
 * 
 * Return value: Resolved enumeration value or NULL.
 **/
FoDatatype*
fo_property_resolve_color_enum (const gchar *token,
				FoContext   *context,
				GError     **error)
{
  g_return_val_if_fail (token != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (strcmp (token, "black") == 0)
    {
      return g_object_ref (fo_color_get_color_black ());
    }
  else if (strcmp (token, "silver") == 0)
    {
      return g_object_ref (fo_color_get_color_silver ());
    }
  else if (strcmp (token, "gray") == 0)
    {
      return g_object_ref (fo_color_get_color_gray ());
    }
  else if (strcmp (token, "white") == 0)
    {
      return g_object_ref (fo_color_get_color_white ());
    }
  else if (strcmp (token, "maroon") == 0)
    {
      return g_object_ref (fo_color_get_color_maroon ());
    }
  else if (strcmp (token, "red") == 0)
    {
      return g_object_ref (fo_color_get_color_red ());
    }
  else if (strcmp (token, "purple") == 0)
    {
      return g_object_ref (fo_color_get_color_purple ());
    }
  else if (strcmp (token, "fuchsia") == 0)
    {
      return g_object_ref (fo_color_get_color_fuchsia ());
    }
  else if (strcmp (token, "green") == 0)
    {
      return g_object_ref (fo_color_get_color_green ());
    }
  else if (strcmp (token, "lime") == 0)
    {
      return g_object_ref (fo_color_get_color_lime ());
    }
  else if (strcmp (token, "olive") == 0)
    {
      return g_object_ref (fo_color_get_color_olive ());
    }
  else if (strcmp (token, "yellow") == 0)
    {
      return g_object_ref (fo_color_get_color_yellow ());
    }
  else if (strcmp (token, "navy") == 0)
    {
      return g_object_ref (fo_color_get_color_navy ());
    }
  else if (strcmp (token, "blue") == 0)
    {
      return g_object_ref (fo_color_get_color_blue ());
    }
  else if (strcmp (token, "teal") == 0)
    {
      return g_object_ref (fo_color_get_color_teal ());
    }
  else if (strcmp (token, "aqua") == 0)
    {
      return g_object_ref (fo_color_get_color_aqua ());
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_ENUMERATION_TOKEN,
		   _(fo_fo_error_messages[FO_FO_ERROR_ENUMERATION_TOKEN]),
		   "color property",
		   token);
      return NULL;
    }
}

/**
 * fo_property_validate_color:
 * @datatype: #FoDatatype to be validated against allowed datatypes and
 *            values for current property
 * @context:  #FoContext object from which to possibly inherit values
 * @error:    Information about any error that has occurred
 * 
 * Validates @datatype against allowed values.  Returns @datatype, a
 * replacement datatype value, or NULL if validation failed
 * 
 * Return value: Valid datatype value or NULL
 **/
FoDatatype*
fo_property_validate_color (FoDatatype *datatype,
			    FoContext  *context,
			    GError    **error)
{
  FoDatatype *new_datatype;
  GError     *tmp_error = NULL;
  gchar      *token;

  g_return_val_if_fail (datatype != NULL, NULL);
  g_return_val_if_fail (FO_IS_DATATYPE (datatype), NULL);
  g_return_val_if_fail (context != NULL, NULL);
  g_return_val_if_fail (FO_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (error == NULL || *error == NULL, NULL);

  if (FO_IS_COLOR (datatype))
    {
      return datatype;
    }
  else if (FO_IS_STRING (datatype))
    {
      token = fo_string_get_value (datatype);

      new_datatype =
	fo_property_resolve_color_enum (token, context, NULL);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else if (FO_IS_NAME (datatype))
    {
      token = fo_name_get_value (datatype);

      new_datatype =
	fo_property_resolve_color_enum (token, context, NULL);

      g_object_unref (datatype);

      if (tmp_error != NULL)
	{
	  g_propagate_error (error, tmp_error);
	  return NULL;
	}

      return new_datatype;
    }
  else
    {
      g_set_error (error,
		   FO_FO_ERROR,
		   FO_FO_ERROR_DATATYPE,
		   _(fo_fo_error_messages[FO_FO_ERROR_DATATYPE]),
		   "color property",
		   fo_object_sprintf (datatype),
		   g_type_name (G_TYPE_FROM_INSTANCE (datatype)));

      g_object_unref (datatype);

      return NULL;
    }
}
